import click
import os
from mpm.model import Plugin
from mpm.config import RepoParamType
import pathlib
from columnar import columnar

@click.group(help='Add and import plugins to repositories')
def repo():
    pass

@repo.command('list', help='List configured repositories')
@click.pass_context
def list_(ctx):
    for repo in ctx.obj['config'].repositories():
        click.echo("{} ({})".format(click.style(repo.name,bold=True), repo.path))
        if not repo.exists():
            click.echo(click.style("\tPath not found: {}".format(repo.path),
                                   fg='red'))
            continue
        rows = []
        currentPlugin = None
        pluginVersions = []
        for plugin in sorted(repo.plugins()):
            if currentPlugin != plugin.name:
                if currentPlugin is not None:
                    rows.append([
                        click.style(currentPlugin, fg='green'),
                        click.style(', '.join(pluginVersions), fg='yellow')
                    ])
                currentPlugin = plugin.name
                pluginVersions = []
            pluginVersions.append(str(plugin.version))
        if currentPlugin is not None:
            rows.append([
                click.style(currentPlugin, fg='green'),
                click.style(', '.join(pluginVersions), fg='yellow')
            ])
        if len(rows) > 0:
            click.echo(columnar(rows, ['Plugin', 'Versions'], no_borders=True))
        else:
            click.echo("No plugins found.")
        for badFile in sorted(repo.badFiles()):
            click.echo('\tWARNING: Unknown file {}'.format(badFile))

@repo.command(help='Add a new repository')
@click.pass_context
@click.argument("name")
@click.argument("path", type=click.Path(path_type=pathlib.Path))
def add(ctx, name, path):
    path.mkdir(parents=True, exist_ok=True)
    ctx.obj['config'].add_repository(name, path)
    ctx.obj['config'].save()
    click.echo("Added repository {} from {}".format(name, path))

@repo.command('import', help='Import a plugin to a repository')
@click.argument('repo', type=RepoParamType())
@click.argument('paths', type=click.Path(exists=True, readable=True), nargs=-1)
def import_(repo, paths):
    if not repo.exists():
        click.echo("Repository '{}' not found at {}".format(repo.name, repo.path))
        return
    plugins = []
    searchPaths = list(paths)
    for path in searchPaths:
        path = pathlib.Path(path)
        try:
            if path.is_dir():
                searchPaths += list(path.glob("*"))
                continue
        except PermissionError:
            click.echo("Cannot read {}".format(path))
            continue
        try:
            plugins.append(Plugin(path))
        except:
            click.echo("Bad plugin filename {}".format(path))

    if len(plugins) == 0:
        click.echo("No plugins found.")
        return

    click.echo('Found the following plugins:')
    for plugin in plugins:
        click.echo("\t{} {}".format(click.style(plugin.name, fg='green'),
                                    click.style(plugin.version, fg='yellow')))
    click.echo("Import plugins into {} ({})? [y/N] ".format(repo.name,
                                                           repo.path), nl=False)
    answer = click.getchar().lower()
    if answer == "y":
        with click.progressbar(plugins, label='Importing') as bar:
            for plugin in bar:
                repo.importPlugin(plugin)
        click.echo("Imported!")
    else:
        click.echo("Cancelled.")
